//******************************************************************************
//Name          : GraphicalAnalysis.cs
//Description   : Implementation to experience the beauty and the pitfalls of Graphical
//                Analysis
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using Display;


class GraphicalAnalysis
{
    public int classes = 15;            // number of classes in histogram

    public int samples;
    public double[] x;                  // signal
    public double[] arg;
    public double[][] histogram;        // 2dim jagged array for histogram
    public double[][] scatterplot;      // 2dim jagged array for scatterplot
    private MyRandom.Random noise;
    private double classwidth;

    // c'tor
    public GraphicalAnalysis(int numofsamples)
    {
        samples = numofsamples;
        x = new double[samples];
        arg = new double[samples];
        histogram = new double[2][];
        histogram[0] = new double[classes];
        histogram[1] = new double[classes];
        scatterplot = new double[2][];
        scatterplot[0] = new double[samples/2];
        scatterplot[1] = new double[samples/2];
        noise = new MyRandom.Random(0.71036, 0.23906);
        classwidth = 0;
    }

    public void CreateSignal(int select)
    {
        if (select == 1)
        {
            // Gleichverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Uniform1();
            }
        }

        if (select == 2)
        {
            // Dreiecksverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Uniform1() + noise.Uniform2();
            }
        }

        if (select == 3)
        {
            // Normalverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Normal(0, 1);
            }
        }

        if (select == 4)
        {
            // Exponentialverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Exponential(1);
            }
        }

        // store argument data
        for (int k = 0; k < samples; k++)
        {
            arg[k] = k;
        }
    }

    public void Histogram(double start, double stop)
    {
        classwidth = (stop - start) / classes;

        // clear old entries if there are any...
        for (int k = 0; k < classes; k++)
        {
            histogram[0][k] = 0;
            histogram[1][k] = 0;
        }

        for (int k=0; k<samples; k++)
        {
            // drop every x(k) in one of the histogram classes
            for (int i=0; i<classes; i++)
            {
                if (x[k] >= (start + i * classwidth) &&
                    x[k] <  (start + (i + 1) * classwidth))
                {
                    histogram[0][i] = start + i*classwidth;
                    histogram[1][i] += 1;
                }
            }
        }

        //// normalizing
        //for (int k=0; k<classes; k++)
        //{
        //    histogram[1][k] /= (double)samples / (double)classes;
        //}
    }

    public void Scatterplot()
    {
        for (int k=0; k<samples; k++)
        {
            if (k % 2 == 0)
            {
                // k even
                scatterplot[0][k/2] = x[k];
            }
            else
            {
                // k uneven (or some other data source!)
                scatterplot[1][(k-1)/2] = x[k];
            }
        }
    }
}


class Execute
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select1 = -1;
        int select2 = -1;

        while (select1 <= 2 && select2 <= 4)
        {
            Console.Write("Grafische Analyse eines linear kongruenten Zufallsgenerators\n");
            Console.WriteLine();
            Console.Write("Analyseverfahren:\n");
            Console.Write("Histogramm     ->  1\n");
            Console.Write("Streudiagramm  ->  2\n");
            userinput = Console.ReadLine();
            select1 = Convert.ToInt32(userinput);

            Console.WriteLine("\n");
            Console.Write("verwendete Testfunktion:\n");
            Console.Write("Gleichverteilung      ->  1\n");
            Console.Write("Dreiecksverteilung    ->  2\n");
            Console.Write("Normalverteilung      ->  3\n");
            Console.Write("Exponentialverteilung ->  4\n");
            Console.WriteLine();
            userinput = Console.ReadLine();
            select2 = Convert.ToInt32(userinput);

            GraphicalAnalysis myAnalysis = new GraphicalAnalysis(1000);
            double xstart = 0;
            double xstop = 0;

            // Darstellung Histogramm
            if (select1 == 1)
            {
                if (select2 == 1)
                {
                    // Gleichverteilung
                    myAnalysis.CreateSignal(1);
                    xstart = 0;
                    xstop = 1;
                }
                else if (select2 == 2)
                {
                    // Dreiecksverteilung
                    myAnalysis.CreateSignal(2);
                    xstart = 0;
                    xstop = 2;
                }
                else if (select2 == 3)
                {
                    // Normalverteilung
                    myAnalysis.CreateSignal(3);
                    xstart = -4;
                    xstop = 4;

                }
                else if (select2 == 4)
                {
                    // Exponentialverteilung
                    myAnalysis.CreateSignal(4);
                    xstart = 0;
                    xstop = 5;
                }

                // fill histogram with data
                myAnalysis.Histogram(xstart, xstop);

                DisplayChart mydisplay1 = new DisplayChart(myAnalysis.arg, myAnalysis.x, "Time domain", "k", "z(k)");
                mydisplay1.Plot();

                DisplayChart mydisplay2 = new DisplayChart(myAnalysis.x, myAnalysis.classes, xstart, xstop, "Histogram", "Interval of x", "Frequency");
                mydisplay2.Hist();
            }

            // Darstellung Streudiagramm
            if (select1 == 2)
            {
                if (select2 == 1)
                {
                    // Gleichverteilung
                    myAnalysis.CreateSignal(1);
                }
                else if (select2 == 2)
                {
                    // Dreiecksverteilung
                    myAnalysis.CreateSignal(2);
                }
                else if (select2 == 3)
                {
                    // Normalverteilung
                    myAnalysis.CreateSignal(3);

                }
                else if (select2 == 4)
                {
                    // Exponentialverteilung
                    myAnalysis.CreateSignal(4);
                }

                // fill scatterplot with data
                myAnalysis.Scatterplot();

                DisplayChart mydisplay1 = new DisplayChart(myAnalysis.arg, myAnalysis.x, "Time domain", "k", "z(k)");
                mydisplay1.Plot();

                DisplayChart mydisplay2 = new DisplayChart(myAnalysis.scatterplot[0], myAnalysis.scatterplot[1], "Scatterplot of z(k)", "k=even", "k=uneven");
                mydisplay2.Scatter();
            }

            for (int i = 0; i < 20; i++) Console.WriteLine();
        }

        Console.WriteLine();
        Console.WriteLine("Ende...<ENTER>");
        Console.ReadLine();
    }
}
